/*-----------------------------------------------------------------------
 *  Torb: an interface between Tcl and CORBA
 *	
 *  Copyright (C) 1999  Colin Hogben
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Author: Colin Hogben <colin.hogben@bigfoot.com>
 *-----------------------------------------------------------------------*/
/*=======================================================================
 *	$Id: torb-orbit.h,v 1.1 1999/06/07 00:43:49 colin Exp $
 *	ORBit-specific macro veneers
 *	These definitions allow the orb-independent torb routines to
 *	generate and call requests using the ORBit orb.
 *=======================================================================*/
#ifndef _torb_orbit_h
#define _torb_orbit_h

#include <orb/orbit.h>

/*--- These should come from a config.h file (platform-dependent) ... */
#define TORB_FLOAT_IS_IEEE 1
#define TORB_DOUBLE_IS_IEEE 1

/*-----------------------------------------------------------------------
 *	Code for marshalling values into a message
 *-----------------------------------------------------------------------*/
#define _TORB_PUTBUF_DECL(m) GIOPSendBuffer *m

#define _TORB_PUTALIGN(m,n) giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(m), n)
#define _TORB_PUTALIGN2(m) _TORB_PUTALIGN(m,2)
#define _TORB_PUTALIGN4(m) _TORB_PUTALIGN(m,4)
#define _TORB_PUTALIGN8(m) _TORB_PUTALIGN(m,8)

#define _TORB__PUTV(m,v) _TORB__PUT(m,&(v),sizeof(v))
#define _TORB__PUT(m,a,s) giop_send_buffer_append_mem_indirect(m, a, s)

/*
 * Marshal integral values.
 * Assume the GIOP_* types are the correct size.
 * Would need to change for platform where e.g. short = 4-bytes.
 */
#define _TORB_MARSHAL_INT1(m,l) \
{ GIOP_octet g = (l); _TORB__PUTV(m,g); }
#define _TORB_MARSHAL_INT2(m,l) \
{ GIOP_short g = (l); _TORB__PUTV(m,g); }
#define _TORB_MARSHAL_INT4(m,l) \
{ GIOP_long g = (l); _TORB__PUTV(m,g); }

/*
 * Marshal real types.
 * This is easy if the platform's float and double use IEEE format,
 * but would require a conversion routine if another format is used.
 */
#if TORB_FLOAT_IS_IEEE
#define _TORB_MARSHAL_FLOAT(m,f) \
{ GIOP_float g = (f); _TORB__PUTV(m,g); }
#else
#error Need float-to-IEEE conversion
#endif
#if TORB_DOUBLE_IS_IEEE
#define _TORB_MARSHAL_DOUBLE(m,d) \
{ GIOP_double g = (d); _TORB__PUTV(m,g); }
#else
#error Need double-to-IEEE conversion
#endif

/*--- Stringy things */
#define _TORB_MARSHAL_STRING(m,s,l) \
{GIOP_unsigned_long sl = ((l)<0 ? strlen(s) : (l))+1; DBG1("string length %lu", sl); _TORB__PUTV(m,sl); _TORB__PUT(m,s,sl);}

/*--- Marshal objects and pseudo-objects */
#define _TORB_MARSHAL_TYPECODE(m,t) ORBit_encode_CORBA_TypeCode(t, m)
#define _TORB_MARSHAL_OBJECT(m,o) ORBit_marshal_object(m,o)


/*-----------------------------------------------------------------------
 *	Code for demarshalling values from a message
 *	*_O versions mean byte order is Ok
 *	*_X versions handle byte swapping (eXchange)
 *-----------------------------------------------------------------------*/
#define _TORB_GETBUF_DECL(m) GIOPRecvBuffer *m

#define _TORB_GETALIGN(m,n) giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(m), n)
#define _TORB_GETALIGN2(m) _TORB_PUTALIGN(m,2)
#define _TORB_GETALIGN4(m) _TORB_PUTALIGN(m,4)
#define _TORB_GETALIGN8(m) _TORB_PUTALIGN(m,8)

#define _TORB_NEED_SWAP(m) giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(m))
#define _TORB__CUR(m) (GIOP_RECV_BUFFER(m)->cur)
#define _TORB__OCUR(m) (*(CORBA_octet **)&_TORB__CUR(m))
#define _TORB__ADV(m,n) (_TORB__OCUR(m) += (n))
#define _TORB__GETOCT(m) (*_TORB__OCUR(m)++)
#define _TORB__GETB(m,a,i) (*((CORBA_octet *)(a)+(i)) = _TORB__GETOCT(m))
#define _TORB__GET2_X(m,a) (_TORB__GETB(m,a,1), _TORB__GETB(m,a,0))
#define _TORB__GET4_X(m,a) (_TORB__GETB(m,a,3), _TORB__GETB(m,a,2), _TORB__GETB(m,a,1), _TORB__GETB(m,a,0))
#define _TORB__GET8_X(m,a) (_TORB__GETB(m,a,7), _TORB__GETB(m,a,6), _TORB__GETB(m,a,5), _TORB__GETB(m,a,4), _TORB__GETB(m,a,3), _TORB__GETB(m,a,2), _TORB__GETB(m,a,1), _TORB__GETB(m,a,0))

#define _TORB__GETV_O(m,t,v) \
{(v) = *(t *)_TORB__CUR(m); _TORB__ADV(m,sizeof(t));}
/* #define _TORB__GETV(m,t,v) {if (_TORB_NEED_SWAP(m)) _TORB__GETV_X(m,t,v); else _TORB__GETV_O(m,t,v);} */


/*--- Demarshal intergral types */
#define _TORB_DEMARSHAL_INT1(m,v) \
((v) = _TORB__GETOCT(m))
#define _TORB_DEMARSHAL_INT2_O(m,v) \
{GIOP_short g; _TORB__GETV_O(m,GIOP_short,g); (v) = g;}
#define _TORB_DEMARSHAL_INT2_X(m,v) \
{GIOP_short g; _TORB__GET2_X(m,&g); (v) = g;}
#define _TORB_DEMARSHAL_INT2(m,v) \
{if (_TORB_NEED_SWAP(m)) {_TORB_DEMARSHAL_INT2_X(m,v);} else {_TORB_DEMARSHAL_INT2_O(m,v);}}
#define _TORB_DEMARSHAL_INT4_O(m,v) \
{GIOP_long g; _TORB__GETV_O(m,GIOP_long,g); (v) = g;}
#define _TORB_DEMARSHAL_INT4_X(m,v) \
{GIOP_long g; _TORB__GET4_X(m,&g); (v) = g;}
#define _TORB_DEMARSHAL_INT4(m,v) \
{if (_TORB_NEED_SWAP(m)) {_TORB_DEMARSHAL_INT4_X(m,v);} else {_TORB_DEMARSHAL_INT4_O(m,v);}}

/*
 * Demarshal real types.
 * This is easy if the platform's float uses IEEE format,
 * but would require a conversion routine if another format is used.
 */
#if TORB_FLOAT_IS_IEEE
#define _TORB_DEMARSHAL_FLOAT_O(m,f) \
{GIOP_float g; _TORB__GETV_O(m,GIOP_float,g); (f) = g;}
#define _TORB_DEMARSHAL_FLOAT_X(m,f) \
{GIOP_float g; _TORB__GET4_X(m,&g); (f) = g;}
#define _TORB_DEMARSHAL_FLOAT(m,f) \
{if (_TORB_NEED_SWAP(m)) {_TORB_DEMARSHAL_FLOAT_X(m,f);} else {_TORB_DEMARSHAL_FLOAT_O(m,f);}}
#else
#error Need IEEE-to-float conversion
#endif

#if TORB_DOUBLE_IS_IEEE
#define _TORB_DEMARSHAL_DOUBLE_O(m,d) \
{GIOP_double g; _TORB__GETV_O(m,GIOP_double,g); (d) = g;}
#define _TORB_DEMARSHAL_DOUBLE_X(m,d) \
{GIOP_float g; _TORB__GET8_X(m,&g); (d) = g;}
#define _TORB_DEMARSHAL_DOUBLE(m,d) \
{if (_TORB_NEED_SWAP(m)) {_TORB_DEMARSHAL_DOUBLE_X(m,d);} else {_TORB_DEMARSHAL_DOUBLE_O(m,d);}}
#else
#error Need IEEE-to-float conversion
#endif

/*--- Stringy things   FIXME optimised version to avoid copying?? */
#define _TORB_DEMARSHAL_STRING(m,s,l) \
{memcpy(s,_TORB__CUR(m),l); _TORB__ADV(m,l);}

/*--- Demarshal objects and pseudo-objects */
#define _TORB_DEMARSHAL_TYPECODE(m,t,orb) \
{ORBit_decode_CORBA_TypeCode(&(t), m);}
#define _TORB_DEMARSHAL_OBJECT(m,o,orb) \
{(o) = ORBit_demarshal_object(msg,orb);}

/*
 * Access typecodes quickly.
 * The 'proper' way is to call CORBA_TypeCode_kind(tc, &ev) etc.
 */
#define _TORB_TYPECODE_KIND(tc) ((tc)->kind)
#define _TORB_TYPECODE_ID(tc) ((tc)->repo_id)
#define _TORB_TYPECODE_NAME(tc) ((tc)->name)
#define _TORB_TYPECODE_MEMBER_COUNT(tc) ((tc)->sub_parts)
#define _TORB_TYPECODE_MEMBER_NAME(tc,i) ((tc)->subnames[i])
#define _TORB_TYPECODE_MEMBER_TYPE(tc,i) ((tc)->subtypes[i])
#define _TORB_TYPECODE_MEMBER_LABEL(tc,i) (&(tc)->sublabels[i])
#define _TORB_TYPECODE_DISCRIMINATOR_TYPE(tc) ((tc)->discriminator)
#define _TORB_TYPECODE_DEFAULT_INDEX(tc) ((tc)->default_index)
#define _TORB_TYPECODE_LENGTH(tc) ((tc)->length)
#define _TORB_TYPECODE_CONTENT_TYPE(tc) ((tc)->subtypes[0])

#endif /* !defined(_torb_orbit_h) */
