/*-----------------------------------------------------------------------
 *  Torb: an interface between Tcl and CORBA
 *	
 *  Copyright (C) 1999  Colin Hogben
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Author: Colin Hogben <colin.hogben@bigfoot.com>
 *-----------------------------------------------------------------------*/
/*=======================================================================
 *	$Id: torb-private.h,v 1.1 1999/06/07 00:43:55 colin Exp $
 *	Common definitions and handy macros for ORBit Tcl binding
 *=======================================================================*/
#ifndef _torb_private_h
#define _torb_private_h

#include <tcl.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "torb.h"

#define ERROR(txt) {Tcl_AppendResult(interp, txt, (char*)0); return TCL_ERROR;}
#define CHK(a) {if ((a) != TCL_OK) return TCL_ERROR;}

/* Protected data */
extern CORBA_ORB torb_orb;
extern Tcl_ObjType torb_TypeCode_objtype;
extern Tcl_ObjType torb_Object_objtype;

/* Protected routines */
extern int torb_init(Tcl_Interp *interp, int argc, char **argv);

/*-----------------------------------------------------------------------
 *	The interface between the front and back ends
 *-----------------------------------------------------------------------*/
extern int torb_do_operation(Tcl_Interp */*interp*/,
			     CORBA_ORB /*orb*/,
			     CORBA_Object /*obj*/,
			     const char */*opname*/,
			     int /*incount*/,
			     CORBA_TypeCode */*intypes*/,
			     Tcl_Obj *CONST */*indata*/,
			     int /*outcount*/,
			     CORBA_TypeCode */*outtypes*/,
			     Tcl_Obj **/*outdata*/,
			     int /*excount*/,
			     CORBA_TypeCode */*extypes*/,
			     const char */*flags*/,
			     ... /*other values dependent on flags*/);

/* Initialise orb if not already done.  Assumes interp */
#define INIT() \
  if (torb_orb == NULL && \
      torb_init(interp, 0, (char **)&interp/*just need valid pointer*/) != TCL_OK) \
    return TCL_ERROR

/* Get string representation of Tcl object */
#define STR(o) Tcl_GetStringFromObj(o, NULL)

/* Check CORBA exception, return with Tcl error if failed.  Assumes interp */
extern int torb_econv(Tcl_Interp */*interp*/, CORBA_Environment */*ev*/);
#define EVRETURN(ev) \
  if ((ev)->_major == CORBA_NO_EXCEPTION) \
  ; else { \
    torb_econv(interp, ev); \
    CORBA_exception_free(ev); \
    return TCL_ERROR; \
  }

/* Make a CORBA call, return with Tcl error if failed.  Assumes interp */
#define EVCHECK(action) \
  { \
    CORBA_Environment ev; \
    CORBA_exception_init(&ev); \
    action; \
    EVRETURN(&ev); \
    CORBA_exception_free(&ev); \
  }

/* Check usage of Tcl command.  Assumes interp, objc, objv, opt */
#define MUSAGE(num,args) MUSAGE2(num,-1,args)
#define MUSAGE2(min,max,args) \
  if (objc < 2+(min) || ((max) >= 0 && objc > 2+(max))) { \
    Tcl_AppendResult(interp, \
		     "usage: ", STR(objv[0]), " ", opt, " ", args, (char*)0); \
    return TCL_ERROR; \
  }

#endif /* !defined(_torb_private_h) */
