/*-----------------------------------------------------------------------
 *  Torb: an interface between Tcl and CORBA
 *	
 *  Copyright (C) 1999  Colin Hogben
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Author: Colin Hogben <colin.hogben@bigfoot.com>
 *-----------------------------------------------------------------------*/
/*=======================================================================
 *	$Id: torb.h,v 1.1 1999/06/07 00:39:22 colin Exp $
 *	Definitions usable by ORB-specific implementation of
 *	torb_do_operation_v() (in torb-<orb>.c) and by Tcl specific
 *	stub files generated by an IDL-compiler
 *=======================================================================*/
#ifndef _torb_h
#define _torb_h

#include <tcl.h>
#include "torb-config.h"

/* Include the ORB-specific definitions, i.e. all _TORB_* macros,
 * and any ORB-specific header files.
 */
#if TORB_USE_ORBIT
#include "torb-orbit.h"
#else
#error No ORB configured
#endif

/*--- TEMPORARY debugging */
#if TORB_DEBUG
#define DBG(s) fprintf(stderr, "%s\n", s)
#define DBG1(f,a) (fprintf(stderr, f, a), DBG(""))
#define DBG2(f,a,b) (fprintf(stderr, f, a, b), DBG(""))
#else
#define DBG(s)
#define DBG1(f,a)
#define DBG2(f,a,b)
#endif

/*-----------------------------------------------------------------------
 *	Extracting values from Tcl objects
 *-----------------------------------------------------------------------*/
/*--- Convert Tcl object to a long integer value */
#define TORB_GET_LONG(i,o,pl,x) \
{if (Tcl_GetLongFromObj(i,o,pl) != TCL_OK) x;}

/*--- Convert Tcl object to a double value */
#define TORB_GET_DOUBLE(i,o,pd,x) \
{if (Tcl_GetDoubleFromObj(i,o,pd) != TCL_OK) x;}

/*--- Optional range checks on short/ushort/octet values */
#if TORB_CHECK_RANGE
#define TORB_RANGE(i,v,a,b,x) \
{if ((v) < (a) || (v) > (b)) x;}
#else /* no range checks */
#define TORB_RANGE(i,v,a,b,x)
#endif

/*-----------------------------------------------------------------------
 *	Marshal individual values.
 *	All indirected via the ORB-specific back end macros
 *-----------------------------------------------------------------------*/
#define TORB_MARSHAL_SHORT(i,m,o,x) \
{long l; TORB_GET_LONG(i,o,&l,x); TORB_RANGE(i,l,-32768,32767,x); _TORB_MARSHAL_INT2(m,l);}
#define TORB_MARSHAL_USHORT(i,m,o,x) \
{long l; TORB_GET_LONG(i,o,&l,x); TORB_RANGE(i,l,0,65535,x); _TORB_MARSHAL_INT2(m,l);}
#define TORB_MARSHAL_LONG(i,m,o,x) \
{long l; TORB_GET_LONG(i,o,&l,x); _TORB_MARSHAL_INT4(m,l);}
#define TORB_MARSHAL_ULONG(i,m,o,x) \
{long l; TORB_GET_LONG(i,o,&l,x); _TORB_MARSHAL_INT4(m,l);}
#define TORB_MARSHAL_FLOAT(i,m,o,x) \
{double d; TORB_GET_DOUBLE(i,o,&d,x); _TORB_MARSHAL_FLOAT(m,d);}
#define TORB_MARSHAL_DOUBLE(i,m,o,x) \
{double d; TORB_GET_DOUBLE(i,o,&d,x); _TORB_MARSHAL_DOUBLE(m,d);}
#define TORB_MARSHAL_BOOLEAN(i,m,o,x) \
{int b; if (Tcl_GetBooleanFromObj(i,o,&b) != TCL_OK) x; _TORB_MARSHAL_INT1(m,b);}
#define TORB_MARSHAL_STRING(i,m,o,x) \
{int len; char *str = Tcl_GetStringFromObj(o,&len); _TORB_MARSHAL_STRING(m,str,len);}
#define TORB_MARSHAL_OBJECT(i,m,o,x) \
{CORBA_Object c; if (torb_CorbaObjGetObject(i,o,&c) != TCL_OK) x; _TORB_MARSHAL_OBJECT(m,c);}
#define TORB_MARSHAL_TYPECODE(i,m,o,x) \
{CORBA_TypeCode t; if (torb_TypeCodeObjGetTc(i,o,&t) != TCL_OK) x; _TORB_MARSHAL_TYPECODE(m,t);}

/*-----------------------------------------------------------------------
 *	Demarshal individual values
 *-----------------------------------------------------------------------*/
#define TORB_DEMARSHAL_SHORT(i,m,po,x) \
{CORBA_short c; _TORB_DEMARSHAL_INT2(m,c); *(po) = Tcl_NewIntObj((int)c);}
#define TORB_DEMARSHAL_USHORT(i,m,po,x) \
{CORBA_unsigned_short c; _TORB_DEMARSHAL_INT2(m,c); *(po) = Tcl_NewIntObj((int)c);}
#define TORB_DEMARSHAL_LONG(i,m,po,x) \
{CORBA_long c; _TORB_DEMARSHAL_INT4(m,c); *(po) = Tcl_NewLongObj((long)c);}
#define TORB_DEMARSHAL_ULONG(i,m,po,x) \
{CORBA_unsigned_long c; _TORB_DEMARSHAL_INT4(m,c); *(po) = Tcl_NewLongObj((long)c);}
#define TORB_DEMARSHAL_FLOAT(i,m,po,x) \
{CORBA_float c; _TORB_DEMARSHAL_FLOAT(m,c); *(po) = Tcl_NewDoubleObj((double)c);}
#define TORB_DEMARSHAL_DOUBLE(i,m,po,x) \
{CORBA_double c; _TORB_DEMARSHAL_DOUBLE(m,c); *(po) = Tcl_NewDoubleObj((double)c);}

/*-----------------------------------------------------------------------
 *	Routines from marshal.c
 *-----------------------------------------------------------------------*/
extern int torb_enum_value(Tcl_Interp */*interp*/,
			   const char */*str*/, CORBA_TypeCode /*tc*/,
			   long */*p_long*/);
extern int torb_marshal_by_typecode(Tcl_Interp */*interp*/,
				    _TORB_PUTBUF_DECL(msg),
				    CORBA_TypeCode /*tc*/, Tcl_Obj */*obj*/);
extern int torb_demarshal_by_typecode(Tcl_Interp */*interp*/,
				      _TORB_GETBUF_DECL(msg),
				      CORBA_TypeCode /*tc*/, Tcl_Obj **/*p_obj*/);

/*-----------------------------------------------------------------------
 *	Routines from object.c and typecode.c
 *-----------------------------------------------------------------------*/
extern Tcl_Obj *torb_NewCorbaObj(Tcl_Interp *interp, CORBA_Object cobj);
extern int torb_CorbaObjGetObject(Tcl_Interp *interp, Tcl_Obj *obj,
				  CORBA_Object *cobjPtr);
extern Tcl_Obj *torb_NewTypeCodeObj(CORBA_TypeCode tc);
extern int torb_TypeCodeObjGetTc(Tcl_Interp *interp, Tcl_Obj *obj,
				 CORBA_TypeCode *tcPtr);

/*-----------------------------------------------------------------------
 *	This supplied by ORB-specific back-end
 *-----------------------------------------------------------------------*/
extern int torb_do_operation_v(Tcl_Interp */*interp*/,
			       CORBA_ORB /*orb*/,
			       CORBA_Object /*obj*/,
			       const char */*opname*/,
			       int /*incount*/,
			       CORBA_TypeCode */*intypes*/,
			       Tcl_Obj *CONST */*indata*/,
			       int /*outcount*/,
			       CORBA_TypeCode */*outtypes*/,
			       Tcl_Obj **/*outdata*/,
			       int /*excount*/,
			       CORBA_TypeCode */*extypes*/,
			       const char */*flags*/,
			       va_list /*other values dependent on flags*/);

/* Values used in flags (unimplemented) */
#define TORB_F_SEND_CONTEXT "c"		/* Pass context: Tcl_Obj* */
#define TORB_F_RECV_CONTEXT "C"		/* Return context: Tcl_Obj** */
#define TORB_F_ONEWAY "1"		/* oneway operation (no arg) */

#endif /* !defined(_torb_h) */
